@@ -3,7 +3,7 @@ from __future__ import annotations
 
 import warnings
 from dataclasses import dataclass, field
-from typing import Any, Dict, Iterable, List, Optional, Tuple
+from typing import Any, Dict, Iterable, List, Optional
 
 import libsbml
 
@@ -99,6 +99,7 @@ class CompoundMixin(BaseModel):
         self.update_meta_info(component="compounds", meta_info={compound: meta_info})
 
     def remove_compound(self, compound: str) -> None:
+        """Remove a compound from the model"""
         self.compounds.remove(compound)
         self._remove_ids([compound])
         try:
@@ -107,13 +108,20 @@ class CompoundMixin(BaseModel):
             pass
 
     def remove_compounds(self, compounds: Iterable[str]) -> None:
+        """Remove compounds from the model"""
         for compound in compounds:
             self.remove_compound(compound=compound)
 
     def _get_all_compounds(self) -> List[str]:
+        """Get all compounds from the model.
+
+        If used together with the algebraic mixin, this will return
+        compounds + derived_compounds. Here it's just for API stability
+        """
         return list(self.compounds)
 
     def get_compounds(self) -> List[str]:
+        """Get the compounds from the model"""
         return list(self.compounds)
 
     ##########################################################################
