@@ -1,13 +1,14 @@
 """Mixin for Parameters."""
 from __future__ import annotations
 
-import json as json
-import pickle as pickle
+import json
+import pickle
 import warnings
 from dataclasses import dataclass, field
-from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple
+from typing import Any, Callable, Dict, Iterable, List, Optional
 
 import libsbml
+from typing_extensions import Literal  # 3.7 compatability
 
 from .basemodel import BaseModel
 from .utils import convert_id_to_sbml, warning_on_one_line
@@ -46,7 +47,7 @@ class ParameterMixin(BaseModel):
 
         Parameters
         ----------
-        meta_info : dict, optional
+        meta_info
             Meta info of the parameter. Allowed keys are
             {unit, database_links, notes}
         """
@@ -62,6 +63,7 @@ class ParameterMixin(BaseModel):
         parameters: Dict[str, float],
         meta_info: Optional[Dict[str, Any]] = None,
     ) -> None:
+        """Add multiple parameters to the model"""
         meta_info = {} if meta_info is None else meta_info
         for parameter_name, parameter_value in parameters.items():
             info = meta_info.get(parameter_name, {})
@@ -72,14 +74,7 @@ class ParameterMixin(BaseModel):
             )
 
     def update_parameter(self, parameter_name: str, parameter_value: float) -> None:
-        """Update a model parameter.
-
-        Parameters
-        ----------
-        parameter_name : str
-            Name of the parameter
-        parameter_value : num
-            Numeric value of the parameter
+        """Update an existing model parameter.
 
         Warns
         -----
@@ -91,12 +86,7 @@ class ParameterMixin(BaseModel):
         self.add_and_update_parameter(parameter_name, parameter_value)
 
     def update_parameters(self, parameters: Dict[str, float]) -> None:
-        """Update existing model parameters.
-
-        Parameters
-        ----------
-        parameters : dict(str: num)
-            Dictionary containing the parameter and value pairs
+        """Update multiple existing model parameters.
 
         See Also
         --------
@@ -117,12 +107,7 @@ class ParameterMixin(BaseModel):
             self._update_derived_parameters()
 
     def add_and_update_parameters(self, parameters: Dict[str, float]) -> None:
-        """Add new and updates existing model parameters.
-
-        Parameters
-        ----------
-        parameters : dict(str: num)
-            Dictionary containing the parameter and value pairs
+        """Add new and update existing model parameters.
 
         See Also
         --------
@@ -136,33 +121,21 @@ class ParameterMixin(BaseModel):
 
         Parameters
         ----------
-        parameter : str
-            Name of the parameter
-        meta_info : dict, optional
+        meta_info
             Meta info of the parameter. Allowed keys are
             {unit, database_links, notes}
         """
         self.update_meta_info(component="parameters", meta_info={parameter: meta_info})
 
     def remove_parameter(self, parameter_name: str) -> None:
-        """Remove a parameter from the model.
-
-        Parameters
-        ----------
-        parameter_name: str
-        """
+        """Remove a parameter from the model."""
         del self.parameters[parameter_name]
         if parameter_name in self.meta_info:
             del self.meta_info["parameters"][parameter_name]
         self._remove_ids([parameter_name])
 
     def remove_parameters(self, parameter_names: Iterable[str]) -> None:
-        """Remove parameters from the model.
-
-        Parameters
-        ----------
-        parameter_names : iterable(str)
-            Names of the parameters that should be removed
+        """Remove multiple parameters from the model.
 
         See Also
         --------
@@ -196,13 +169,7 @@ class ParameterMixin(BaseModel):
         )
 
     def remove_derived_parameter(self, parameter_name: str) -> None:
-        """Remove a derived parameter from the model.
-
-        Parameters
-        ----------
-        parameter_name : str
-            Name of the derived parameter
-        """
+        """Remove a derived parameter from the model."""
         old_parameter = self.derived_parameters.pop(parameter_name)
         derived_from = old_parameter["parameters"]
         for i in derived_from:
@@ -226,16 +193,12 @@ class ParameterMixin(BaseModel):
                 update_derived=False,
             )
 
-    def store_parameters_to_file(self, filename: str, filetype: str = "json") -> None:
-        """Store the parameters into a json or pickle file.
-
-        Parameters
-        ----------
-        filename : str
-            The name of the pickle file
-        filetype : {json, pickle}
-            Output file type.
-        """
+    def store_parameters_to_file(
+        self,
+        filename: str,
+        filetype: Literal["json", "pickle"] = "json",
+    ) -> None:
+        """Store the parameters into a json or pickle file."""
         if filetype == "json":
             if not filename.endswith(".json"):
                 filename += ".json"
@@ -249,16 +212,12 @@ class ParameterMixin(BaseModel):
         else:
             raise ValueError("Can only save to json or pickle")
 
-    def load_parameters_from_file(self, filename: str, filetype: str = "json") -> None:
-        """Load parameters from a json or pickle file.
-
-        Parameters
-        ----------
-        filename : str
-            The name of the pickle file
-        filetype : {json, pickle}
-            Input file type.
-        """
+    def load_parameters_from_file(
+        self,
+        filename: str,
+        filetype: Literal["json", "pickle"] = "json",
+    ) -> None:
+        """Load parameters from a json or pickle file."""
         if filetype == "json":
             with open(filename, "r") as f:
                 self.add_and_update_parameters(parameters=json.load(f))
@@ -273,30 +232,15 @@ class ParameterMixin(BaseModel):
         self.parameters = self.initialization_parameters.copy()
 
     def get_parameter(self, parameter_name: str) -> float:
-        """Return a single parameter.
-
-        Parameters
-        ----------
-        parameter_name : str
-            Name of the parameter
-
-        Returns
-        -------
-        parameter_value : float
-            Value of the parameter
-        """
+        """Return the value of a single parameter."""
         return float(self.parameters[parameter_name])
 
     def get_parameters(self) -> Dict[str, float]:
-        """Return all parameters.
-
-        Returns
-        -------
-        parameters : dict(str, num)
-        """
+        """Return all parameters."""
         return dict(self.parameters)
 
     def get_parameter_names(self) -> List[str]:
+        """Return names of all parameters"""
         return list(self.parameters)
 
     ##########################################################################
