@@ -23,6 +23,8 @@ warnings.formatwarning = warning_on_one_line  # type: ignore
 
 
 class Rate:
+    """A container for a rate"""
+
     def __init__(
         self,
         name: str,
@@ -61,11 +63,15 @@ class Rate:
 
     def _check_rate_args(self) -> None:
         difference = set(self.args).difference(self.dynamic_variables + self.parameters)
+        if self.reversible:
+            targets = "substrates, products, modifiers or parameters"
+        else:
+            targets = "substrates, modifiers or parameters"
         if difference:
-            targets = "substrates, products or modifiers or parameters"
             warnings.warn(f"Supplied args {difference} for rate {self.name} that aren't in {targets}")
 
-    def _check_consistency(self) -> None:
+    def check_consistency(self) -> None:
+        """Check if args and arity of the rate are ok."""
         self._check_rate_dynamic_variables()
         self._check_rate_args()
         if not check_function_arity(function=self.function, arity=len(self.args)):
@@ -78,6 +84,7 @@ class Rate:
         return f"{self.function.__name__}({', '.join(self.args)}) -> {', '.join(self.products)}"
 
     def keys(self) -> List[str]:
+        """Get all valid keys of the rate"""
         return [
             "function",
             "substrates",
@@ -103,6 +110,7 @@ class Rate:
         yield "reversible", self.reversible
 
     def copy(self) -> Rate:
+        """Create a copy of the rate"""
         return copy.deepcopy(self)
 
 
@@ -136,28 +144,34 @@ class RateMixin(ParameterMixin, CompoundMixin, BaseModel):
     ##########################################################################
 
     def add_function(self, function_name: str, function: Callable[..., float]) -> None:
+        """Add a function to the model"""
         if function.__name__ == "<lambda>":
             patch_lambda_function_name(function=function, name=function_name)
 
         self.functions[function_name] = function
 
     def add_functions(self, functions: Dict[str, Callable[..., float]]) -> None:
+        """Add multiple functions to the model"""
         for function_name, function in functions.items():
             self.add_function(function_name=function_name, function=function)
 
     def update_function(self, function_name: str, function: Callable[..., float]) -> None:
+        """Update a function"""
         if function.__name__ == "<lambda>":
             patch_lambda_function_name(function=function, name=function_name)
         self.functions[function_name] = function
 
     def update_functions(self, functions: Dict[str, Callable[..., float]]) -> None:
+        """Update multiple functions"""
         for function_name, function in functions.items():
             self.update_function(function_name, function)
 
     def remove_function(self, function_name: str) -> None:
+        """Remove a function from the model"""
         del self.functions[function_name]
 
     def remove_functions(self, function_names: List[str]) -> None:
+        """Remove multiple functions from the model"""
         for function_name in function_names:
             self.remove_function(function_name)
 
@@ -174,7 +188,7 @@ class RateMixin(ParameterMixin, CompoundMixin, BaseModel):
             candidates=rate.parameters,
             element_type="parameter",
         )
-        rate._check_consistency()
+        rate.check_consistency()
 
     def add_rate(
         self,
@@ -401,6 +415,7 @@ class RateMixin(ParameterMixin, CompoundMixin, BaseModel):
             self._check_rate_consistency(rate)
 
     def update_rates(self, rates: dict) -> None:
+        """Update multiple rates."""
         for rate_name, rate in rates.items():
             self.update_rate(rate_name, **rate)
 
@@ -443,6 +458,7 @@ class RateMixin(ParameterMixin, CompoundMixin, BaseModel):
         return list(self.rates)
 
     def get_rate_function(self, rate_name: str) -> Callable[..., float]:
+        """Get the function of a rate."""
         return self.rates[rate_name].function
 
     def get_rate_parameters(self, rate_name: str) -> List[str]:
@@ -485,7 +501,7 @@ class RateMixin(ParameterMixin, CompoundMixin, BaseModel):
             try:
                 fluxes[name] = float(rate.function(*(args[arg] for arg in rate.args)))
             except KeyError as e:
-                raise KeyError(f"Could not find argument {e} for rate {name}")
+                raise KeyError(f"Could not find argument {e} for rate {name}") from e
         return fluxes
 
     def _get_fluxes_array(
@@ -500,7 +516,7 @@ class RateMixin(ParameterMixin, CompoundMixin, BaseModel):
             try:
                 fluxes[name] = np.atleast_1d(rate.function(*(args[arg] for arg in rate.args)))
             except KeyError as e:
-                raise KeyError(f"Could not find argument {e} for rate {name}")
+                raise KeyError(f"Could not find argument {e} for rate {name}") from e
         return fluxes
 
     ##########################################################################
