@@ -4,8 +4,7 @@ from __future__ import annotations
 import itertools as it
 import warnings
 from collections import defaultdict
-from dataclasses import dataclass
-from typing import Any, Callable, Dict, List, Optional, Tuple, Union, cast
+from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union, cast
 
 import libsbml
 import numpy as np
@@ -74,8 +73,7 @@ class LabelModel(Model):
         """
         if num_labels > 0:
             return [base_name + "__" + "".join(i) for i in it.product(("0", "1"), repeat=num_labels)]
-        else:
-            return [base_name]
+        return [base_name]
 
     def add_compound(  # type: ignore
         self,
@@ -334,9 +332,9 @@ class LabelModel(Model):
         for compound in compounds:
             try:
                 labels_per_compound.append(self.label_compounds[compound]["num_labels"])
-            except KeyError:
+            except KeyError as e:
                 if compound not in self.get_compounds():
-                    raise KeyError(f"Compound {compound} neither a compound nor a label compound")
+                    raise KeyError(f"Compound {compound} neither a compound nor a label compound") from e
                 labels_per_compound.append(0)
         return labels_per_compound
 
@@ -450,8 +448,7 @@ class LabelModel(Model):
                 for label_pos in external_labels:
                     external_label_string[label_pos] = "1"
             return "".join(external_label_string)
-        else:
-            return ""
+        return ""
 
     def _add_base_reaction(
         self,
@@ -459,8 +456,8 @@ class LabelModel(Model):
         rate_name: str,
         function: Callable,
         stoichiometry: Dict[str, int],
-        base_substrates: List[str],
-        base_products: List[str],
+        # base_substrates: List[str],
+        # base_products: List[str],
         labelmap: List[int],
         external_labels: Optional[str],
         modifiers: Optional[List[str]],
@@ -655,8 +652,8 @@ class LabelModel(Model):
         self._add_base_reaction(
             rate_name=rate_name,
             function=function,
-            base_substrates=base_substrates,
-            base_products=base_products,
+            # base_substrates=base_substrates,
+            # base_products=base_products,
             stoichiometry=stoichiometry,
             labelmap=labelmap,
             external_labels=external_label_str,
@@ -797,7 +794,7 @@ class LabelModel(Model):
         rate_base_name: str,
         y: Union[Dict[str, float], Dict[str, List[float]]],
         t: Union[float, List[float]] = 0,
-    ) -> pd.DataFrame:
+    ) -> np.ndarray:
         """Get total fluxes of a base rate.
 
         Parameters
@@ -811,7 +808,7 @@ class LabelModel(Model):
         fluxes : numpy.array
         """
         rates = [i for i in self.rates if i.startswith(rate_base_name + "__")]
-        return self.get_fluxes_df(y=y, t=t)[rates].sum(axis=1).values
+        return cast(np.ndarray, self.get_fluxes_df(y=y, t=t)[rates].sum(axis=1).values)
 
     def _create_label_scope_seed(
         self, *, initial_labels: Union[Dict[str, int], Dict[str, List[int]]]
@@ -888,19 +885,18 @@ class LabelModel(Model):
                             new_cpds[cpd] = 1
             # Isolate "old" labels
             s1 = pd.Series(labelled_compounds)
-            s1 = s1[s1 == 1]
+            s1 = cast(pd.Series, s1[s1 == 1])
             # Isolate new labels
             s2 = pd.Series(new_cpds)
-            s2 = s2[s2 == 1]
+            s2 = cast(pd.Series, s2[s2 == 1])
             # Find new labels
-            new_labels = set(s2.index).difference(set(s1.index))
+            new_labels = cast(Set[str], set(s2.index).difference(set(s1.index)))
             # Break the loop once no new labels can be found
             if new_labels == set():
                 break
-            else:
-                labelled_compounds = new_cpds
-                result[loop_count] = new_labels
-                loop_count += 1
+            labelled_compounds = new_cpds
+            result[loop_count] = new_labels
+            loop_count += 1
         return result
 
     ##########################################################################
@@ -910,8 +906,7 @@ class LabelModel(Model):
     def _map_label_compound_to_compound(self, *, compound: str) -> str:
         if compound in self.nonlabel_compounds:
             return compound
-        else:
-            return compound.rsplit("__")[0]
+        return compound.rsplit("__")[0]
 
     def to_model(self) -> Model:
         """Convert LabelModel to Model."""
