@@ -1,14 +1,13 @@
 """Abstract base class for Simulator interfaces."""
 from __future__ import annotations
 
-import concurrent.futures as futures
 import copy
-import itertools as it
-import json as json
-import pickle as pickle
-import sys as sys
+import json
+import pickle
+import sys
 import warnings
 from abc import ABC, abstractmethod
+from concurrent import futures
 from functools import partial
 from typing import (
     Any,
@@ -20,7 +19,6 @@ from typing import (
     Optional,
     Tuple,
     Type,
-    TypeVar,
     Union,
     cast,
 )
@@ -83,6 +81,7 @@ class _BaseSimulator(Generic[BASE_MODEL_TYPE], ABC):
         self.integrator = self._integrator(rhs=self.model._get_rhs, y0=y0)
 
     def get_integrator_params(self) -> Optional[Dict[str, Any]]:
+        """Some docstring"""
         if self.integrator is None:
             return None
         return self.integrator.get_integrator_kwargs()
@@ -97,28 +96,27 @@ class _BaseSimulator(Generic[BASE_MODEL_TYPE], ABC):
         split_array: List[np.ndarray],
         normalise: Union[float, List[float]],
     ) -> List[np.ndarray]:
+        """Some docstring"""
         if isinstance(normalise, (int, float)):
             return [i / normalise for i in split_array]
-        else:  # iterable
-            # not concatenated
-            if len(normalise) == len(split_array):
-                return [i / np.reshape(j, (len(i), 1)) for i, j in zip(split_array, normalise)]
-            # concatenated
-            else:
-                results = []
-                start = 0
-                end = 0
-                for i in split_array:
-                    end += len(i)
-                    results.append(i / np.reshape(normalise[start:end], (len(i), 1)))
-                    start += end
-                return results
+        if len(normalise) == len(split_array):
+            return [i / np.reshape(j, (len(i), 1)) for i, j in zip(split_array, normalise)]
+
+        results = []
+        start = 0
+        end = 0
+        for i in split_array:
+            end += len(i)
+            results.append(i / np.reshape(normalise[start:end], (len(i), 1)))
+            start += end
+        return results
 
     @abstractmethod
     def _test_run(self) -> None:
         """Perform a test step of the simulation in Python to get proper error handling."""
 
     def _save_simulation_results(self, *, time: np.ndarray, results: np.ndarray, skipfirst: bool) -> None:
+        """Some docstring"""
         if self.time is None or self.results is None:
             self.time = [time]
             self.results = [results]
@@ -141,8 +139,7 @@ class _BaseSimulator(Generic[BASE_MODEL_TYPE], ABC):
             return None
         if concatenated:
             return np.concatenate(self.time, axis=0)  # type: ignore
-        else:
-            return self.time.copy()
+        return self.time.copy()
 
     def simulate(
         self,
@@ -232,29 +229,22 @@ class _BaseSimulator(Generic[BASE_MODEL_TYPE], ABC):
             results = self._normalise_split_array(split_array=results, normalise=normalise)
         if concatenated:
             return np.concatenate(results, axis=0)  # type: ignore
-        else:
-            return results
+        return results
 
     def get_results_dict(
         self,
         normalise: Union[float, List[float]] = None,
         concatenated: bool = True,
     ) -> Union[None, Dict[str, np.ndarray], List[Dict[str, np.ndarray]]]:
-        """Get simulation results.
-
-        Returns
-        -------
-        results : dict
-        """
+        """Get simulation results."""
         results = self.get_results_array(normalise=normalise, concatenated=concatenated)
         if results is None:
             return None
         if concatenated:
             results = cast(np.ndarray, results)  # this is just a typing annotation
             return dict(zip(self.model.get_compounds(), results.T))
-        else:
-            results = cast(List[np.ndarray], results)  # this is just a typing annotation
-            return [dict(zip(self.model.get_compounds(), i.T)) for i in results]
+        results = cast(List[np.ndarray], results)  # this is just a typing annotation
+        return [dict(zip(self.model.get_compounds(), i.T)) for i in results]
 
     def get_results_df(
         self,
@@ -272,15 +262,14 @@ class _BaseSimulator(Generic[BASE_MODEL_TYPE], ABC):
                 index=self.get_time(),
                 columns=self.model.get_compounds(),
             )
-        else:
-            return [
-                pd.DataFrame(
-                    data=result,
-                    index=t,
-                    columns=self.model.get_compounds(),
-                )
-                for t, result in zip(time, results)
-            ]
+        return [
+            pd.DataFrame(
+                data=result,
+                index=t,
+                columns=self.model.get_compounds(),
+            )
+            for t, result in zip(time, results)
+        ]
 
     def store_results_to_file(self, filename: str, filetype: str = "json") -> None:
         """Store the simulation results into a json or pickle file.
@@ -483,18 +472,19 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         return time, results
 
     def _calculate_fluxes(self) -> None:
-        fluxes = []
         if any((i is None for i in [self.time, self.results, self.simulation_parameters])):
-            return None
-        for t, y, pars in zip(
-            cast(List[np.ndarray], self.time),
-            cast(List[np.ndarray], self.results),
-            cast(List[Dict[str, float]], self.simulation_parameters),
-        ):
-            self.update_parameters(parameters=pars)
-            fluxes_array = self.model.get_fluxes_array(y=y, t=t)
-            fluxes.append(fluxes_array)
-        self.fluxes = fluxes
+            pass
+        else:
+            fluxes = []
+            for t, y, pars in zip(
+                cast(List[np.ndarray], self.time),
+                cast(List[np.ndarray], self.results),
+                cast(List[Dict[str, float]], self.simulation_parameters),
+            ):
+                self.update_parameters(parameters=pars)
+                fluxes_array = self.model.get_fluxes_array(y=y, t=t)
+                fluxes.append(fluxes_array)
+            self.fluxes = fluxes
 
     def get_fluxes_array(
         self,
@@ -511,8 +501,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             fluxes = self._normalise_split_array(split_array=fluxes, normalise=normalise)
         if concatenated:
             return np.concatenate(fluxes, axis=0)  # type: ignore
-        else:
-            return fluxes
+        return fluxes
 
     def get_fluxes_dict(
         self,
@@ -525,8 +514,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             return None
         if concatenated:
             return dict(zip(self.model.rates, cast(np.ndarray, fluxes).T))
-        else:
-            return [dict(zip(self.model.rates, i.T)) for i in fluxes]
+        return [dict(zip(self.model.rates, i.T)) for i in fluxes]
 
     def get_fluxes_df(
         self,
@@ -544,15 +532,14 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
                 index=self.get_time(),
                 columns=self.model.get_rate_names(),
             )
-        else:
-            return [
-                pd.DataFrame(
-                    data=flux,
-                    index=t,
-                    columns=self.model.get_rate_names(),
-                )
-                for t, flux in zip(time, fluxes)
-            ]
+        return [
+            pd.DataFrame(
+                data=flux,
+                index=t,
+                columns=self.model.get_rate_names(),
+            )
+            for t, flux in zip(time, fluxes)
+        ]
 
     def _calculate_full_results(self) -> None:
         full_results = []
@@ -586,8 +573,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             )
         if concatenated:
             return np.concatenate(full_results, axis=0)  # type: ignore
-        else:
-            return full_results
+        return full_results
 
     def get_full_results_dict(
         self,
@@ -601,8 +587,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         all_compounds = self.model.get_all_compounds()
         if concatenated:
             return dict(zip(all_compounds, cast(np.ndarray, full_results).T))
-        else:
-            return [dict(zip(all_compounds, i.T)) for i in full_results]
+        return [dict(zip(all_compounds, i.T)) for i in full_results]
 
     def get_full_results_df(
         self,
@@ -618,10 +603,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
 
         if concatenated:
             return pd.DataFrame(data=full_results, index=time, columns=all_compounds)
-        else:
-            return [
-                pd.DataFrame(data=res, index=t, columns=all_compounds) for t, res in zip(time, full_results)
-            ]
+        return [pd.DataFrame(data=res, index=t, columns=all_compounds) for t, res in zip(time, full_results)]
 
     def get_variable(
         self,
@@ -640,8 +622,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             return None
         if concatenated:
             return cast(Dict[str, np.ndarray], full_results_dict)[variable]
-        else:
-            return [i[variable] for i in cast(List[Dict[str, np.ndarray]], full_results_dict)]
+        return [i[variable] for i in cast(List[Dict[str, np.ndarray]], full_results_dict)]
 
     def get_variables(
         self,
@@ -654,9 +635,8 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         if concatenated:
             full_results_df = cast(pd.DataFrame, full_results_df)
             return full_results_df.loc[:, variables].values  # type: ignore
-        else:
-            full_results_df = cast(List[pd.DataFrame], full_results_df)
-            return [i.loc[:, variables].values for i in full_results_df]
+        full_results_df = cast(List[pd.DataFrame], full_results_df)
+        return [i.loc[:, variables].values for i in full_results_df]
 
     @staticmethod
     def _parameter_scan_worker(
@@ -687,16 +667,18 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         concentrations = dict(s.get_full_results_df(concatenated=True).iloc[-1])  # type: ignore
         return parameter_value, concentrations, fluxes
 
-    def _scan_threading(self, parameter_values: list[float], worker: Callable) -> list:
-        with tqdm(total=len(parameter_values)) as pbar:
+    def _scan_threading(self, parameter_name: str, parameter_values: list[float], worker: Callable) -> list:
+        with tqdm(total=len(parameter_values), desc=parameter_name) as pbar:
             results = []
             for value in parameter_values:
                 results.append(worker(value))
                 pbar.update(1)
         return results
 
-    def _scan_multiprocessing(self, parameter_values: list[float], worker: Callable) -> list:
-        with tqdm(total=len(parameter_values)) as pbar:
+    def _scan_multiprocessing(
+        self, parameter_name: str, parameter_values: list[float], worker: Callable
+    ) -> list:
+        with tqdm(total=len(parameter_values), desc=parameter_name) as pbar:
             with futures.ProcessPoolExecutor() as executor:
                 results = []
                 for task in futures.as_completed((executor.submit(worker, i) for i in parameter_values)):
@@ -733,9 +715,9 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             },
         )
         if sys.platform in ["win32", "cygwin"] or not multiprocessing:
-            results = self._scan_threading(parameter_values, worker)
+            results = self._scan_threading(parameter_name, parameter_values, worker)
         else:
-            results = self._scan_multiprocessing(parameter_values, worker)
+            results = self._scan_multiprocessing(parameter_name, parameter_values, worker)
         concentrations = {}
         for (i, conc, _) in results:
             concentrations[i] = conc
@@ -748,8 +730,14 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         tolerance: float = 1e-8,
         multiprocessing: bool = True,
         **integrator_kwargs: Dict[str, Any],
-    ) -> Union[pd.DataFrame, Tuple[pd.DataFrame, pd.DataFrame]]:
+    ) -> Tuple[pd.DataFrame, pd.DataFrame]:
         """Scan the model steady state changes caused by a change to a parameter."""
+        if sys.platform in ["win32", "cygwin"]:
+            warnings.warn(
+                """
+                Windows does not behave well with multiple processes.
+                Falling back to threading routine."""
+            )
         worker = partial(
             self._parameter_scan_worker,
             **{
@@ -764,21 +752,9 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             },
         )
         if sys.platform in ["win32", "cygwin"] or not multiprocessing:
-            warnings.warn(
-                """
-                Windows does not behave well with multiple processes.
-                Falling back to threading routine."""
-            )
-            Executor = futures.ThreadPoolExecutor  # type: ignore
+            results = self._scan_threading(parameter_name, parameter_values, worker)
         else:
-            Executor = futures.ProcessPoolExecutor  # type: ignore
-        with tqdm(total=len(parameter_values)) as pbar:
-            with Executor() as executor:
-                tasks = [executor.submit(worker, i) for i in parameter_values]
-                results = []
-                for task in futures.as_completed(tasks):
-                    results.append(task.result())
-                    pbar.update(1)
+            results = self._scan_multiprocessing(parameter_name, parameter_values, worker)
         concentrations = {}
         fluxes = {}
         for (i, conc, flux) in results:
@@ -786,6 +762,59 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             fluxes[i] = flux
         return pd.DataFrame(concentrations).T.sort_index(), pd.DataFrame(fluxes).T.sort_index()
 
+    def parameter_scan_2d(
+        self,
+        p1: Tuple[str, List[float]],
+        p2: Tuple[str, List[float]],
+        tolerance: float = 1e-8,
+        multiprocessing: bool = True,
+        **integrator_kwargs: Dict[str, Any],
+    ) -> Dict[float, pd.DataFrame]:
+        """Some docstring"""
+        cs = {}
+        parameter_name1, parameter_values1 = p1
+        parameter_name2, parameter_values2 = p2
+        original_pars = self.model.parameters.copy()
+        for value in tqdm(parameter_values2, total=len(parameter_values2), desc=parameter_name2):
+            self.update_parameter(parameter_name2, value)
+            cs[value] = self.parameter_scan(
+                parameter_name1,
+                parameter_values1,
+                tolerance=tolerance,
+                multiprocessing=multiprocessing,
+                **integrator_kwargs,
+            )
+        self.update_parameters(original_pars)
+        return cs
+
+    def parameter_scan_2d_with_fluxes(
+        self,
+        p1: Tuple[str, List[float]],
+        p2: Tuple[str, List[float]],
+        tolerance: float = 1e-8,
+        multiprocessing: bool = True,
+        **integrator_kwargs: Dict[str, Any],
+    ) -> Tuple[Dict[float, pd.DataFrame], Dict[float, pd.DataFrame]]:
+        """Some docstring"""
+        cs = {}
+        vs = {}
+        parameter_name1, parameter_values1 = p1
+        parameter_name2, parameter_values2 = p2
+        original_pars = self.model.parameters.copy()
+        for value in tqdm(parameter_values2, total=len(parameter_values2), desc=parameter_name2):
+            self.update_parameter(parameter_name2, value)
+            c, v = self.parameter_scan_with_fluxes(
+                parameter_name1,
+                parameter_values1,
+                tolerance=tolerance,
+                multiprocessing=multiprocessing,
+                **integrator_kwargs,
+            )
+            cs[value] = c
+            vs[value] = v
+        self.update_parameters(original_pars)
+        return cs, vs
+
     def plot_log(
         self,
         xlabel: str = None,
@@ -804,6 +833,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         compounds = self.model.get_compounds()
         x = cast(np.ndarray, self.get_time(concatenated=True))
         y = cast(pd.DataFrame, self.get_full_results_df(normalise=normalise, concatenated=True))
@@ -850,6 +880,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         compounds = self.model.get_compounds()
         x = cast(np.ndarray, self.get_time(concatenated=True))
         y = cast(pd.DataFrame, self.get_full_results_df(normalise=normalise, concatenated=True))
@@ -899,6 +930,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         compounds = self.model.get_derived_compounds()
         x = cast(np.ndarray, self.get_time(concatenated=True))
         y = cast(pd.DataFrame, self.get_full_results_df(normalise=normalise, concatenated=True))
@@ -941,6 +973,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         compounds = self.model.get_all_compounds()
         x = cast(np.ndarray, self.get_time(concatenated=True))
         y = cast(pd.DataFrame, self.get_full_results_df(normalise=normalise, concatenated=True))
@@ -984,6 +1017,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         x = cast(np.ndarray, self.get_time(concatenated=True))
         y = cast(pd.DataFrame, self.get_full_results_df(normalise=normalise, concatenated=True))
         if x is None or y is None:
@@ -1020,7 +1054,6 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         figure_title: str = None,
         grid: bool = True,
         tight_layout: bool = True,
-        ax: plt.Axes = None,
         figure_kwargs: Dict[str, Any] = None,
         subplot_kwargs: Dict[str, Any] = None,
         plot_kwargs: Dict[str, Any] = None,
@@ -1053,7 +1086,6 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             plot_titles=plot_titles,
             grid=grid,
             tight_layout=tight_layout,
-            ax=ax,
             figure_kwargs=figure_kwargs,
             subplot_kwargs=subplot_kwargs,
             plot_kwargs=plot_kwargs,
@@ -1082,6 +1114,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         if xlabel is None:
             xlabel = variable
         results = cast(pd.DataFrame, self.get_full_results_df(concatenated=True))
@@ -1127,6 +1160,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         if xlabel is None:
             xlabel = variable
         results = cast(pd.DataFrame, self.get_full_results_df(concatenated=True))
@@ -1172,6 +1206,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         if xlabel is None:
             xlabel = variable
         results = cast(pd.DataFrame, self.get_full_results_df(concatenated=True))
@@ -1218,6 +1253,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         if xlabel is None:
             xlabel = variable
         results = cast(pd.DataFrame, self.get_full_results_df(concatenated=True))
@@ -1262,6 +1298,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         rate_names = cast(List[str], self.model.get_rate_names())
         x = self.get_time()
         y = self.get_fluxes_df(normalise=normalise, concatenated=True)
@@ -1306,6 +1343,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         x = self.get_time()
         y = self.get_fluxes_df(normalise=normalise, concatenated=True)
         if x is None or y is None:
@@ -1343,7 +1381,6 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         figure_title: str = None,
         grid: bool = True,
         tight_layout: bool = True,
-        ax: plt.Axes = None,
         figure_kwargs: Dict[str, Any] = None,
         subplot_kwargs: Dict[str, Any] = None,
         plot_kwargs: Dict[str, Any] = None,
@@ -1383,7 +1420,6 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
             plot_titles=plot_titles,
             grid=grid,
             tight_layout=tight_layout,
-            ax=ax,
             figure_kwargs=figure_kwargs,
             subplot_kwargs=subplot_kwargs,
             plot_kwargs=plot_kwargs,
@@ -1412,6 +1448,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         if xlabel is None:
             xlabel = variable
         rate_names = cast(List[str], self.model.get_rate_names())
@@ -1458,6 +1495,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         if xlabel is None:
             xlabel = variable
         x = self.get_variable(variable=variable)
@@ -1503,6 +1541,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         if xlabel is None:
             xlabel = cpd1
         if ylabel is None:
@@ -1551,6 +1590,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         kwargs = _get_plot_kwargs(
             figure_kwargs=figure_kwargs,
             subplot_kwargs=subplot_kwargs,
@@ -1617,6 +1657,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         xlabel = cpd1 if xlabel is None else xlabel
         ylabel = cpd2 if ylabel is None else ylabel
 
@@ -1689,6 +1730,7 @@ class _BaseRateSimulator(Generic[RATE_MODEL_TYPE], _BaseSimulator[RATE_MODEL_TYP
         label_kwargs: Dict[str, Any] = None,
         title_kwargs: Dict[str, Any] = None,
     ) -> Tuple[Optional[plt.Figure], Optional[plt.Axes]]:
+        """Some docstring"""
         kwargs = _get_plot_kwargs(
             figure_kwargs=figure_kwargs,
             subplot_kwargs=subplot_kwargs,
