@@ -1,8 +1,10 @@
+"""Some docstring"""
+
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from collections import defaultdict
-from typing import Callable, Dict, List, Tuple, Union
+from typing import Callable, Dict, List, Union
 
 from . import ratefunctions
 
@@ -17,6 +19,8 @@ def _pack_stoichiometries(substrates: List[str], products: List[str]) -> Dict[st
 
 
 class BaseRateLaw(ABC):
+    """Some docstring"""
+
     def __init__(self) -> None:
         self.substrates: List[str] = []
         self.products: List[str] = []
@@ -35,6 +39,8 @@ class BaseRateLaw(ABC):
 
 
 class Constant(BaseRateLaw):
+    """Some docstring"""
+
     def __init__(self, product: str, k: str) -> None:
         super().__init__()
         self.products = [product]
@@ -50,6 +56,8 @@ class Constant(BaseRateLaw):
 
 
 class MassAction(BaseRateLaw):
+    """Some docstring"""
+
     def __init__(
         self,
         substrates: Union[str, List[str]],
@@ -75,6 +83,8 @@ class MassAction(BaseRateLaw):
 
 
 class ReversibleMassAction(BaseRateLaw):
+    """Some docstring"""
+
     def __init__(
         self,
         substrates: Union[str, List[str]],
@@ -110,6 +120,8 @@ class ReversibleMassAction(BaseRateLaw):
 
 
 class MichaelisMenten(BaseRateLaw):
+    """Some docstring"""
+
     def __init__(self, S: str, P: str, vmax: str, km: str) -> None:
         super().__init__()
         self.substrates = [S]
@@ -129,6 +141,8 @@ class MichaelisMenten(BaseRateLaw):
 
 
 class ReversibleMichaelisMenten(BaseRateLaw):
+    """Some docstring"""
+
     def __init__(self, S: str, P: str, vmax_fwd: str, vmax_bwd: str, km_fwd: str, km_bwd: str) -> None:
         super().__init__()
         self.substrates = [S]
@@ -155,6 +169,8 @@ class ReversibleMichaelisMenten(BaseRateLaw):
 
 
 class ReversibleMichaelisMentenKeq(BaseRateLaw):
+    """Some docstring"""
+
     def __init__(self, S: str, P: str, vmax_fwd: str, km_fwd: str, km_bwd: str, k_eq: str) -> None:
 
         super().__init__()
